define(['jquery'], function ($) {
    'use strict';
    var SidebarMixin = {

        options: {
            isRecursive: true,
            minicart: {
                maxItemsVisible: 3
            },
            qty: {
                minQty: 1,
                maxQty: 100,
                increment: 1
            },
            item: {
                'qty': ':input.cart-item-qty',
                'button': ':button.update-cart-item',
                'inputQty': '.item-qty.cart-item-qty',
                'qtyIncrement': '.qty-button.qty-increment',
                'qtyDecrement': '.qty-button.qty-decrement',
                'updateButton': 'update-cart-item',
            },
        },

        /**
         * Update sidebar block.
         */
        update: function () {
            this._super();
            this._checkMaxAndMinusButtons();
        },

        /**
         * @private
         */
        _initContent: function () {
            this._super();
            let new_events = {};
            let self = this;

            /**
             * @param {jQuery.Event} event
             */
            new_events['click ' + this.options.item.qtyIncrement] = function (event) {
                let variation = self.options.qty.increment;
                self._adjustQty($(event.currentTarget), variation);
                event.stopPropagation();
                self._autoUpdateItemQty($(event.currentTarget));
            };

            /**
             * @param {jQuery.Event} event
             */
            new_events['click ' + this.options.item.qtyDecrement] = function (event) {
                let variation = self.options.qty.increment * -1;
                self._adjustQty($(event.currentTarget), variation);
                event.stopPropagation();
                self._autoUpdateItemQty($(event.currentTarget));
            };

            /**
             * @param {jQuery.Event} event
             */
            new_events['change ' + this.options.item.inputQty] = function (event) {
                self._autoUpdateItemQty($(event.currentTarget));
            };

            this._on(this.element, new_events);
        },

        _checkMaxAndMinusButtons: function () {
            let self = this,
                target = $(this.options.item.inputQty);
            
            target.each(function(index, elm) {
                let itemId = $(elm).data('cart-item');

                self._checkAvailability(itemId);
            });
        },

        /**
         * @param {HTMLElement} elem
         * @private
         */
        _autoUpdateItemQty: function (elem) {
            var itemId = elem.data('minicart-item');

            this._ajax(this.options.url.update, {
                'item_id': itemId,
                'item_qty': $('#minicart-item-' + itemId + '-qty').val()
            }, elem, this._updateItemQtyAfter);
        },

        /**
         * @param {HTMLElement} elem
         * @param {Integer} variation
         * @private
         */
        _adjustQty: function(elem, variation) {
            let itemId = elem.data('minicart-item');
            let currentQty = parseInt($('#minicart-item-' + itemId + '-qty').val());

            if(isNaN(currentQty)) {
                currentQty = 0;
            } else {
                currentQty = Math.floor((currentQty + variation) / variation) * variation;
            }

            if(currentQty <= this.options.qty.minQty) {

                currentQty = this.options.qty.minQty;

            } else if(currentQty >= this.options.qty.maxQty){

                currentQty = this.options.qty.maxQty;

            } else if(currentQty < Math.abs(variation)) {

                currentQty = Math.abs(variation);
            }

            $('#minicart-item-' + itemId + '-qty').val(currentQty);
            this._checkAvailability(itemId);
        },

        _checkAvailability: function(itemId) {
            let qty = parseInt($('#minicart-item-' + itemId + '-qty').val());

            $('#minicart-item-' + itemId + '-qty').parent().find('.qty-decrement, .qty-increment').removeClass('disabled');

            if (qty <= this.options.qty.minQty){
                $('#minicart-item-' + itemId + '-qty').parent().find('.qty-decrement').addClass('disabled');
            }
            if (qty >= this.options.qty.maxQty){
                $('#minicart-item-' + itemId + '-qty').parent().find('.qty-increment').addClass('disabled');
            }
        },

    };

    return function (targetWidget) {
        $.widget('mage.sidebar', targetWidget, SidebarMixin);

        return $.mage.sidebar;
    };

});
